/*


  MakeWDG - Concatination of files into .WDG Warzone resource files
    - Parses .WRF files


  - Tim Cannell - Pumpkin Studios - EIDOS - 1997


  v0.1 . First beta version
  v0.1a  Pad each file with 1 byte
  v0.1b  Added new data types
  v0.1c  new padding so each file is on long work boundry
  v0.1d  new data types and expanded areas
  v0.2	 now writes correct area sizes without parameter setting (using temp file) (WDG2) - also no filename text area by default
  v0.2a	 - verbose option  - fix header size bug
  v0.2b  - Add animation types ANI - ANIMCFG
  V0.2c  - Added BPIE 
  v0.3	- Needed to aligned data to a long word boundry
  v0.3a	- added SWEAPSND		
  v0.3b - added TXR_RES - binary text file format !
  v0.3c - added BLO - binary script format
  v0.4	- -i option added for installing "IMD" files as binary types (and so getting the files from .bin directories)
  v0.4a - -b option added for installing "SCRIPT" files as binary types (using .SLO extensions from .bin directory) - Also removed .BLO type
  v0.4b -  added SWEAPMOD & SWEAPSND types
  v0.4c - removed SPROGRAM & STEMPPROG - added SPROPSND
  v0.4d - added SSTRMOD
  v0.4e	- added TEXPAGE24 & IMGCLUTHB
  v0.5  - new wdg version (3) with trimmed directory 
 
  v1.0  - now combined wrf & wdg with multiple wrf combined into 1 wdg (cool eh!)
  v1.1  - now adds a zero to the end of every file
  v1.2  - binary scripts & pie fixed
  v1.3	- fixed bug in verbose mode

*/					   // printf

#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <assert.h>

#include "frame.h"
#include "wdg.h"

#define MAX_STR (256)

static	FILE *WDGfile=NULL;

static	FILE *TMPfile=NULL;
typedef BOOL (*SEARCHCALLBACK)(char *FilePath,void *UserData);


UDWORD CurrentWDGoffset=0;

static BOOL Verbose=FALSE;


int CopyTempData(void);

BOOL AddWRFEntry(char *type, char *name);



WRFINFO WRFinfo[MAXFILESINWRF];
WDGINFO WDGinfo[MAXWRFINWDG];



char WRFnames[MAXWRFINWDG][MAX_STR];	// array of names of wrfs


WDG_HEADER Header;

WDG_HEADER_V5	V5Header;

int WRFcount;	// how many wrfs to process


			   	char Drive[16];
   		char Dir[MAX_PATH];
   		char FName[MAX_PATH];
   		char Ext[MAX_PATH];
   		char Pattern[MAX_PATH];
   		char Path[MAX_PATH];



/***************************************************************************/
BOOL UseSubdirect=FALSE;
int AlignValue=0;
char WRFfilename[MAX_STR]="test.wrf";
char WDGfilename[MAX_STR]="test.wdg";
char DataDirectory[MAX_STR]="";
										 


BOOL BinaryIMD=FALSE;
BOOL BinarySCRIPT=FALSE;
BOOL AddonWDG=FALSE;

UDWORD fileoffset,FileRoundValue;
UDWORD filenumber;		// number of files in the current wrf

#define MAXMISC (32)	// max number of 32 misc data wildcard entries

UDWORD MiscDataCount;
char MiscDataFiles[MAXMISC][255];


// WDG dependancies for this WDG
#define MAXDEP	20
UDWORD	numDeps;
WDG_DEP	aDeps[MAXDEP];

// the name/sequence number of this WDG
UDWORD	WDGsequence;
STRING	aWDGName[WDG_NAMEMAX];

BOOL CaseSensitive=FALSE;		// FALSE converts all names to lowercase
/***************************************************************************/




void Usage(void)
{
	printf("Usage: MakeWDG -o test.wdg -a wrf\*.* test1.wrf test2.wrf test3.wrf\n");

		printf("-m = display message of the day\n");

		printf("-o = output .wdg file [%s]\n",WDGfilename);
		printf("-d = data directory [%s]\n",DataDirectory);

		printf("-i = Get IMD's from .BIN directorys\n");
		printf("-b = Get SCRIPT's from .BIN directorys\n");

		printf("-w = Use this wildcard pattern of wrf's\n");
		printf("-v = verbose\n");

		printf("-a = Add a file\wildcard to the MISCDATA section\n");

		printf("-x <sequence> <name> = Create an addon WDG");

		printf("-s <sequence> <name> = specify a WDG that this WDG depends on");

		printf("\n\nMakeWDG V1.3 - %s %s\n",__DATE__,__TIME__);

		printf("    Pumpkin Studios 1997/8/9 - Tim Cannell\n");

}
				   



int WriteHeader(void)
{
	UDWORD PaddingGap,Offset;

	
	if (WDGsequence == 0)
	{
		// writing full WDG

		Header.WDGid[0]='W';
		Header.WDGid[1]='D';
		Header.WDGid[2]='G';
		Header.WDGid[3]='4';		// now version 4 - new multiple wrf's
		Header.NumWRFFiles=WRFcount;

		fwrite(&Header,sizeof(WDG_HEADER),1,WDGfile);

		// now write out the WRF infomation

		fwrite(WDGinfo,WRFcount*sizeof(WDGINFO),1,WDGfile);	// write out the data to the WDG file

		return(sizeof(WDG_HEADER) + (WRFcount*sizeof(WDGINFO)));
	}
	else
	{
		// writing new partial WDG
		V5Header.WDGid[0]='W';
		V5Header.WDGid[1]='D';
		V5Header.WDGid[2]='G';
		V5Header.WDGid[3]='5';		// now version 4 - new multiple wrf's
		V5Header.NumWRFFiles=WRFcount;
		strcpy(V5Header.aName, aWDGName);
		V5Header.sequence = WDGsequence;
		V5Header.numDep = numDeps;

		fwrite(&V5Header,sizeof(WDG_HEADER_V5),1,WDGfile);

		// now write out the dependancy info
		if (numDeps > 0)
		{
			fwrite(aDeps,sizeof(WDG_DEP),numDeps,WDGfile);
		}

		// now write out the WRF infomation

		fwrite(WDGinfo,WRFcount*sizeof(WDGINFO),1,WDGfile);	// write out the data to the WDG file

		return(sizeof(WDG_HEADER_V5) + sizeof(WDG_DEP) * numDeps + (WRFcount*sizeof(WDGINFO)));
	}

}



int WriteWRFHeader(void)
{
	int headersize;

	fwrite(WRFinfo,filenumber*sizeof(WRFINFO),1,WDGfile);	// write out the data to the WDG file

	return filenumber*sizeof(WRFINFO);
}



void ReadMOTD(void)
{
	char String[4096];
	int ret;

	char MOTDfilename[]={"\\\\SPEED\\MOTD\\MOTD.TXT"};

	
	
	
	TMPfile=fopen(MOTDfilename,"rt");	
	if (TMPfile==NULL)
	{
		printf("Unable to open motd file\n");
		return;
	}

	while(ret!=EOF)
	{
		ret=fgetc(TMPfile);
		if (ret!=EOF) putchar(ret);

	}
	fclose(TMPfile);
}



BOOL WritePad(FILE *padfile, int padsize)
{
#define bufsize 512
	char buffer[bufsize];
	if (padsize<=0) return TRUE;	// nothing to do
	
	memset(buffer,0x00,bufsize);

	while(padsize>=bufsize)
	{
		fwrite(buffer,bufsize,1,padfile);
		padsize-=bufsize;
	}

	fwrite(buffer,padsize,1,padfile);

	return TRUE;
}

BOOL ProcessFile(char *FilePath,void *UserData)
{

	AddWRFEntry("MISCDATA", FilePath, "");
	if (Verbose) printf("miscdata adding %s\n",FilePath);

	return FALSE;
}

BOOL WRFProcessFile(char *FilePath,void *UserData)
{
	strcpy(WRFnames[WRFcount],FilePath);
	if (Verbose) 	printf("adding %s\n",WRFnames[WRFcount]);
	WRFcount++;
	return FALSE;
}



void main(int argc, char *argv[] )
{
	int CurrentArg;
	BOOL result;
	BOOL StartReadWRFs;
	int CurrentWRFNumber;	// which WRF in the wdg are we processing
	int headersize;
	int tempsize;

	WIN32_FIND_DATA FindData;
	HANDLE hSearch;
	BOOL res;

	// Handle the options

	FileRoundValue=0;		// how much do we round the file by

	CurrentArg=1;	// Start at arg 1, arg 0 is the command name

	MiscDataCount=0;

	StartReadWRFs=FALSE;	// read the '-' arguments first
	while (CurrentArg < argc)
	{
		if (argv[CurrentArg][0]!='-')
		{
			StartReadWRFs=TRUE;
		}


		if (StartReadWRFs==FALSE)
		{

			switch (tolower(argv[CurrentArg][1]))
			{
				case 'm':
					ReadMOTD();
					break;

				case 'd':		// set data directory
					if (argv[CurrentArg][2]!=0)
					{
						strcpy(DataDirectory,&argv[CurrentArg][2]);
					}
					else
					{
						CurrentArg++;
						strcpy(DataDirectory,argv[CurrentArg]);
					}
					break;
				case 'o':		// Define .WDG file
					if (argv[CurrentArg][2]!=0)
					{
						strcpy(WDGfilename,&argv[CurrentArg][2]);
					}
					else
					{
						CurrentArg++;
						strcpy(WDGfilename,argv[CurrentArg]);
					}
					break;
				case 'i':
					BinaryIMD=TRUE;
					break;
				case 'b':
					BinarySCRIPT=TRUE;
					break;

				case 'v':
					Verbose=TRUE;
					break;

				case 'x':
					AddonWDG = TRUE;

					// get the sequence number
					if (argv[CurrentArg][2]!=0)
					{
						WDGsequence=atoi(& argv[CurrentArg][2]);
					}
					else
					{
						CurrentArg++;
						WDGsequence=atoi(argv[CurrentArg]);
					}

					// get the WDG name
					CurrentArg++;
					strcpy(aWDGName, argv[CurrentArg]);
					break;

				// Add a miscdata wildcard/file thingy
				case 'a':
					CurrentArg++;
					strcpy(MiscDataFiles[MiscDataCount],argv[CurrentArg]);
					MiscDataCount++;
					break;
				case 'w':
					// Parse it as a wildcard thingy
					CurrentArg++;

   				_splitpath(argv[CurrentArg],Drive,Dir,FName,Ext);

				sprintf(Pattern,"%s%s",FName,Ext);
				sprintf(Path,"%s%s",Drive,Dir);
				RecurseDirectories(TRUE,Path,"",Pattern,WRFProcessFile,NULL,NULL);	// thanks paul
/*

					hSearch=FindFirstFile(argv[CurrentArg],&FindData);
					if (hSearch==INVALID_HANDLE_VALUE)
					{
						printf("No matching files\n");
						Usage();
						exit(-1);
					}
					res=TRUE;
					while(res==TRUE)
					{

						strcpy(WRFnames[WRFcount],FindData.cFileName);
						printf("adding %s\n",WRFnames[WRFcount]);
						WRFcount++;
						res=FindNextFile(hSearch,&FindData);
					}
*/
					break;
				

				case 'r':
					if (argv[CurrentArg][2]!=0)
					{
						FileRoundValue=atoi(& argv[CurrentArg][2]);
					}
					else
					{
						CurrentArg++;
						FileRoundValue=atoi(argv[CurrentArg]);
					}
					break;

				case 's':
					if (numDeps >= MAXDEP)
					{
						printf("Too many dependancies\n");
						exit(-1);
					}
					if (argv[CurrentArg][2]!=0)
					{
						aDeps[numDeps].sequence=atoi(& argv[CurrentArg][2]);
					}
					else
					{
						CurrentArg++;
						aDeps[numDeps].sequence=atoi(argv[CurrentArg]);
					}

					// get the WDG name
					CurrentArg++;
					strcpy(aDeps[numDeps].aName, argv[CurrentArg]);

					numDeps ++;
					break;

				default:
					Usage();
					exit(-1);
					break;
			}
	
		}
		else
		{
			// read in a wrf name
			strcpy(WRFnames[WRFcount],argv[CurrentArg]);
			WRFcount++;
		}

		CurrentArg++;
	}
	// do we have any misc data to add
	if (MiscDataCount>0)
	{
		strcpy(WRFnames[WRFcount],"MISCDATA");
		WRFcount++;
	}

	_chdir(DataDirectory);

	resInitialise();


	if (BinaryIMD==TRUE)
		AddBinaryResourceType("IMD");

	if (BinarySCRIPT==TRUE)
		AddBinaryResourceType("SCRIPT");

	CurrentWRFNumber=0;		// which wrf are are looking at

	WDGfile=fopen(WDGfilename,"wb");	
	if (WDGfile==NULL)
	{
		printf("Unable to open file [%s] as a WDG file\n",WDGfilename);
		resShutDown();
		exit(-1);
	}

	CurrentWDGoffset=0;
	// write the wdg headers  ... with dummy null data
	headersize=WriteHeader();			

	CurrentWDGoffset+=headersize;	
/*
	if (FileRoundValue>0)
	{
		int padsize;
		padsize=FileRoundValue-(headersize&(FileRoundValue-1));
		WritePad(WDGfile,padsize);
		CurrentWDGoffset+=padsize;	
	}
*/	

	for (CurrentWRFNumber=0;CurrentWRFNumber<WRFcount;CurrentWRFNumber++)
	{

		char wrfentryname[256];
		int pos;
		// start of individual wrf parsing
	
		strcpy(WRFfilename,WRFnames[CurrentWRFNumber]);


		if (Verbose) printf("Parsing %s...\n",WRFfilename);


		// get the .wrf name from the filename

		pos=strlen(WRFfilename);
		while (pos>0)
		{
			if (WRFfilename[pos]=='\\')
			{
				pos++;		// we need to skip the "\"
				break;
			}

			pos--;
		}
		strcpy(wrfentryname,&WRFfilename[pos]);




		TMPfile=fopen("WDG.tmp","wb+");	
		if (TMPfile==NULL)
		{
			printf("Unable to create temporary file [WDG.tmp]\n");
			resShutDown();
			exit(-1);
		}


		resSetPreLoadCallback(AddWRFEntry);
		fileoffset=0;				// offset into file data this file is stored at
		filenumber=0;				// which file in the wdg are we processing

		if (strcmp(wrfentryname,"MISCDATA")==0)
		{
			UDWORD Entry;

// Now we need to traverse all the miscdata stuff adding them to the WDG
			for (Entry=0;Entry<MiscDataCount;Entry++)
			{
				/*
					Possible correct strings

					gamedesc.lev
					sequenceaudio\c001.txt
					wrf\*.*
				*/
   				_splitpath(MiscDataFiles[Entry],Drive,Dir,FName,Ext);

				sprintf(Pattern,"%s%s",FName,Ext);
				sprintf(Path,"%s%s",Drive,Dir);
				RecurseDirectories(TRUE,Path,"",Pattern,ProcessFile,NULL,NULL);	// thanks paul
			}		
		}
		else
		{

			result=resLoad(WRFfilename,0,0,0,0);	// parse the .wrf files and build the wdg file
			if (result==FALSE)
			{
				printf("We encountered some problems whilst parsing this .WRF file\n ... sorry about that!\n");
			}
		}	

		WDGinfo[CurrentWRFNumber].WRFname=HashStringIgnoreCase(wrfentryname);
		WDGinfo[CurrentWRFNumber].offset=CurrentWDGoffset;
		WDGinfo[CurrentWRFNumber].filecount=filenumber;
		
		if (Verbose) printf("entry = [%s] hash=%x\n",wrfentryname,WDGinfo[CurrentWRFNumber].WRFname);
		
		headersize=WriteWRFHeader();			//	write the header for this wrf
		CurrentWDGoffset+=headersize;	
/*
		if (FileRoundValue>0)
		{
			int padsize;
			padsize=FileRoundValue-(headersize&(FileRoundValue-1));
			WritePad(WDGfile,padsize);
			CurrentWDGoffset+=padsize;	
		}
*/
		// copy all the wrf's datas
		tempsize=CopyTempData();				// copy data from the temporary file to the WDG
		CurrentWDGoffset+=tempsize;	
	
		fclose(TMPfile);
	}

	fseek(WDGfile,0,SEEK_SET);	// go back to the front of the file

	// write the wdg headers  ... 
	WriteHeader();				 // rewrite the header with the correct info in it

	fclose(WDGfile);

	// stats
	printf("%d WRFs loaded\n",WRFcount);

	printf("bye bye\n");
	resShutDown();
}

#define BUFSIZE (4096)
char zerobuffer[]={0,0,0,0,0,0,0,0};


BOOL AddWRFEntry(char *type, char *name, char *in_dir)
{
	UDWORD TypeHash,NameHash;
	FILE *file;
	UDWORD roundedFileSize;
	UDWORD LoadedSize,PadAmount;

	UBYTE SoftwareFlag;
	SBYTE TexturePage;

	char buffer[BUFSIZE];
	char filename[256];
	char dir[256];
	fpos_t filesize;

	assert(BUFSIZE>=FileRoundValue);	

	strcpy(dir,in_dir);

	if (BinaryIMD==TRUE)
	{
		if (strcmp(type,"IMD")==0)
		{
			if (strlen(dir)>=1)	dir[strlen(dir)-1]=0;	// shorten string by 1 character to remove slash
			strcat(dir,".bin\\");
			if (Verbose) printf("file %s (type %s) is processed as binary from directory %s\n",name,type,dir);
		}
	}

	if (BinarySCRIPT==TRUE)
	{
		if (strcmp(type,"SCRIPT")==0)
		{
			if (strlen(dir)>=1) dir[strlen(dir)-1]=0;	// shorten string by 1 character to remove slash
			strcat(dir,".bin\\");
			if (Verbose) printf("file %s (type %s) is processed as binary from directory %s\n",name,type,dir);
		}
	}



	TypeHash=HashString(type);
	NameHash=HashStringIgnoreCase(name);

	if (Verbose) printf("%d) offset=%d type=[%s]:$%x   name=[%s]:$%x   dir=[%s]\n",filenumber,fileoffset,type,TypeHash,name,NameHash,dir);

	strcpy(filename,dir);
	strcat(filename,name);

	ScanString(name,&SoftwareFlag,&TexturePage);

	file=fopen(filename,"rb");

	if ((file==NULL) && !AddonWDG)
	{
		printf("unable to load [%s] !!\n",filename);
		return FALSE;
	}

	if (file != NULL)
	{
		fseek(file,0,SEEK_END);

		fgetpos(file,&filesize);

		fseek(file,0,SEEK_SET);

		do
		{
			LoadedSize=fread(buffer,1,BUFSIZE,file);
			fwrite(buffer,LoadedSize,1,TMPfile);	// write out the data to the WDG file
		} while (LoadedSize==BUFSIZE);


		filesize++;	
		fwrite(zerobuffer,1,1,TMPfile);	// write out a zero termination
		


		fclose(file);



		if (FileRoundValue>0)
		{
			PadAmount= (FileRoundValue - (filesize&(FileRoundValue-1))) &(FileRoundValue-1);

			memset(buffer,0xaa,PadAmount);
			fwrite(buffer,PadAmount,1,TMPfile);

			roundedFileSize=(filesize+FileRoundValue-1)&(~(FileRoundValue-1));
		}
		else
		{
			roundedFileSize=filesize;
		}

		if (Verbose) printf("filesize=%d\n",filesize);
		if (Verbose) printf("padamount=%d\n",PadAmount);
		if (Verbose) printf("roundedfilesize=%d\n",roundedFileSize);
		if (Verbose) printf("fileroundvalue=%d\n",FileRoundValue);
		
		// Write out the WRF header
		WRFinfo[filenumber].type=TypeHash;
		WRFinfo[filenumber].name=NameHash;
		WRFinfo[filenumber].offset=fileoffset;
		WRFinfo[filenumber].filesize=filesize;
		
		WRFinfo[filenumber].SoftwareFlag=SoftwareFlag;
		WRFinfo[filenumber].TexturePage=TexturePage;

		if (Verbose &&(SoftwareFlag!=0 || TexturePage!=-1 ))
		{
			printf("Texture page string =[%s]\n",name);
			printf("Software flag=%d\n",SoftwareFlag);
			printf("Texture page = %d\n",TexturePage);
		}


		filenumber++;
		fileoffset+=roundedFileSize;
	}
	else if (AddonWDG)
	{
		if (Verbose)
		{
			printf("File not found, making pass thru entry for %s\n", name);
		}

		// Write out the WRF header
		WRFinfo[filenumber].type=TypeHash;
		WRFinfo[filenumber].name=NameHash;
		WRFinfo[filenumber].offset=UDWORD_MAX;
		WRFinfo[filenumber].filesize=UDWORD_MAX;
		
		WRFinfo[filenumber].SoftwareFlag=SoftwareFlag;
		WRFinfo[filenumber].TexturePage=TexturePage;

		if (Verbose &&(SoftwareFlag!=0 || TexturePage!=-1 ))
		{
			printf("Texture page string =[%s]\n",name);
			printf("Software flag=%d\n",SoftwareFlag);
			printf("Texture page = %d\n",TexturePage);
		}


		filenumber++;
	}

	return FALSE;		// false indicates not to contiune loading the file
}


int CopyTempData(void)
{
	char buffer[BUFSIZE];
	int LoadedSize;
	int totalwritten=0;

	fseek(TMPfile,0,SEEK_SET);	// goto start of tmp file

	do
	{
		LoadedSize=fread(buffer,1,BUFSIZE,TMPfile);
		fwrite(buffer,LoadedSize,1,WDGfile);	// write out the data to the WDG file
		totalwritten+=LoadedSize;
	} while (LoadedSize==BUFSIZE);

	if (Verbose) printf("total data copied = %d\n",totalwritten);
	return totalwritten;
}

//

// Recursivly scan through the specified directory path.
// Stolen from paul ...
//
BOOL RecurseDirectories(BOOL Recurse,char *RootPath,char *SourceName,char *MatchName,
						SEARCHCALLBACK Callback,void *UserData,FILE *Stream)
{
	WIN32_FIND_DATA FileData;
	HANDLE Handle;
	char FullPath[MAX_PATH];
	char CurrFile[MAX_PATH];
	BOOL Found;
	BOOL Return = FALSE;

// List files in this group.

	// handle a null root
	if(RootPath[0]==0)
	{
		if(SourceName[0]) 
		{
			sprintf(FullPath,"%s\\%s",SourceName,MatchName);
		}
		else
		{
			sprintf(FullPath,"%s",MatchName);
		}
	}
	else
	{
		if(SourceName[0]) 
		{
			sprintf(FullPath,"%s\\%s\\%s",RootPath,SourceName,MatchName);
		}
		else
		{
			sprintf(FullPath,"%s\\%s",RootPath,MatchName);
		}
	}
	Found = TRUE;
	Handle = FindFirstFile(FullPath,&FileData);
	if(Handle == INVALID_HANDLE_VALUE) {
		Found = FALSE;
	}

	while(Found) {
		if(FileData.cFileName[0] != '.') {

			if(SourceName[0]) 
			{
//				sprintf(CurrFile,"%s\\%s\\%s",RootPath,SourceName,FileData.cFileName);
				sprintf(CurrFile,"%s%s\\%s",RootPath,SourceName,FileData.cFileName);
			}
			else
			{
//				sprintf(CurrFile,"%s\\%s%s",RootPath,SourceName,FileData.cFileName);
				sprintf(CurrFile,"%s%s%s",RootPath,SourceName,FileData.cFileName);
			}

			if(GetFileAttributes(CurrFile) != FILE_ATTRIBUTE_DIRECTORY) {
				if(SourceName[0]) 
				{
//					sprintf(CurrFile,"%s\\%s\\%s",RootPath,SourceName,FileData.cFileName);
					sprintf(CurrFile,"%s%s\\%s",RootPath,SourceName,FileData.cFileName);
				} 
				else
				{
//					sprintf(CurrFile,"%s\\%s%s",RootPath,SourceName,FileData.cFileName);
					sprintf(CurrFile,"%s%s%s",RootPath,SourceName,FileData.cFileName);
				}

				// Do the file found call back.
				if(Callback) {
					if(Callback(CurrFile,UserData) == TRUE) {
						FindClose(Handle);
						return TRUE;
					}
				}
			}
		}

		Found = FindNextFile(Handle,&FileData);
	}

	FindClose(Handle);

	if(Recurse) {
	// Now recurse for groups in this group.

		sprintf(FullPath,"%s\\%s\\*.*",RootPath,SourceName);

		Handle = FindFirstFile(FullPath,&FileData);
		if(Handle == INVALID_HANDLE_VALUE) {
			return FALSE;
		}

		Found = TRUE;

		while(Found) {
			if(FileData.cFileName[0] != '.') {

				if(SourceName[0]) {
					sprintf(CurrFile,"%s\\%s\\%s",RootPath,SourceName,FileData.cFileName);
				} else {
					sprintf(CurrFile,"%s\\%s%s",RootPath,SourceName,FileData.cFileName);
				}

				if(GetFileAttributes(CurrFile) == FILE_ATTRIBUTE_DIRECTORY) {

					if(SourceName[0]) {
						sprintf(CurrFile,"%s\\%s",SourceName,FileData.cFileName);
					} else {
						sprintf(CurrFile,"%s%s",SourceName,FileData.cFileName);
					}

					Return = RecurseDirectories(TRUE,RootPath,CurrFile,MatchName,Callback,UserData,Stream);
					if(Return) break;
				}
			}

			Found = FindNextFile(Handle,&FileData);
		}

		FindClose(Handle);
	}

	return Return;
}




